@@ -86,6 +86,7 @@ STATIC_VAR heBrowser_s *heHelpBrowsers=NULL;
  * Implementation: public function
  *
  *****************************************************************/
+EXTERN_VAR BOOLEAN FE_OPT_NO_SHELL_FLAG;
 void feHelp(char *str)
 {
   str = strclean(str);
@@ -818,7 +819,7 @@ static BOOLEAN heGenInit(int warn, int br)
     {
       case '#': break;
       case ' ': break;
-      case 'i': /* singular.hlp */
+      case 'i': /* singular.info */
       case 'x': /* singular.idx */
       case 'h': /* html dir */
                if (feResource(*p, warn) == NULL)
@@ -852,7 +853,7 @@ static BOOLEAN heGenInit(int warn, int br)
 
                  if ((op=='O') && (strcmp(name,S_UNAME)!=0))
                    return FALSE;
-                 if ((op=='E') && (omFindExec(name,exec)==NULL))
+                 if ((op=='E') && ((omFindExec(name,exec)==NULL) || FE_OPT_NO_SHELL_FLAG))
                  {
                    if (warn) Warn("executable `%s` not found",name);
                    return FALSE;
@@ -913,32 +914,55 @@ static void heGenHelp(heEntry hentry, int br)
                    else
                    {
                      if (*p=='H')
+                     #ifdef SINGULAR_4_2
+                       snprintf(temp,256,"%s/%d-%d/index.htm", htmldir,
+                                  SINGULAR_VERSION/1000,
+                                 (SINGULAR_VERSION % 1000)/100
+                       );
+                     #else
                        snprintf(temp,256,"%s/%d-%d-%d/index.htm", htmldir,
                                   SINGULAR_VERSION/1000,
                                  (SINGULAR_VERSION % 1000)/100,
                                  (SINGULAR_VERSION % 100)/10
                        );
+                     #endif
                      else
                        snprintf(temp,256,"%s/index.htm", htmldir);
                    }
                    #else
                    {
                      if (*p=='H')
+                     #ifdef SINGULAR_4_2
+                       sprintf(temp,"%s/%d-%d/%s", htmldir,
+                                  SINGULAR_VERSION/1000,
+                                 (SINGULAR_VERSION % 1000)/100,
+                       hentry->url);
+                     else
+                       sprintf(temp,"%s/%s", htmldir, hentry->url);
+                     #else
                        sprintf(temp,"%s/%d-%d-%d/%s", htmldir,
                                   SINGULAR_VERSION/1000,
                                  (SINGULAR_VERSION % 1000)/100,
                                  (SINGULAR_VERSION % 100)/10,
                        hentry->url);
                      else
-                       sprintf(temp,"%s/%d-%d-%d/%s", htmldir, hentry->url);
+                       sprintf(temp,"%s/%s", htmldir, hentry->url);
+                     #endif
                    }
                    else
                      if (*p=='H')
+                     #ifdef SINGULAR_4_2
+                       sprintf(temp,"%s/%d-%d/index.htm", htmldir,
+                                  SINGULAR_VERSION/1000,
+                                 (SINGULAR_VERSION % 1000)/100
+                       );
+                     #else
                        sprintf(temp,"%s/%d-%d-%d/index.htm", htmldir,
                                   SINGULAR_VERSION/1000,
                                  (SINGULAR_VERSION % 1000)/100,
                                  (SINGULAR_VERSION % 100)/10
                        );
+                     #endif
                      else
                        sprintf(temp,"%s/index.htm", htmldir);
                    }
@@ -957,13 +981,13 @@ static void heGenHelp(heEntry hentry, int br)
                    i=strlen(sys);
                    break;
                  }
-        case 'i': /* singular.hlp */
+        case 'i': /* singular.info */
                  {
                    char *i_res=feResource('i');
                    if (i_res!=NULL) strcat(sys,i_res);
                    else
                    {
-                     WarnS("singular.hlp not found");
+                     WarnS("singular.info not found");
                      return;
                    }
                    i=strlen(sys);
@@ -1003,7 +1027,7 @@ static void heGenHelp(heEntry hentry, int br)
     }
   }
   Print("running `%s`\n",sys);
-  (void) system(sys);
+  if (!FE_OPT_NO_SHELL_FLAG) (void) system(sys);
 }
 
 static BOOLEAN heDummyInit(int /*warn*/, int /*br*/)
@@ -1027,12 +1051,12 @@ static void heEmacsHelp(heEntry hentry, int /*br*/)
   WarnS("to enter the Singular online help. For general");
   WarnS("information on Singular running under Emacs, type C-h m.");
 }
-static int singular_manual(char *str, BOOLEAN isIndexEntry);
+static int singular_manual(char *str, BOOLEAN isIndexEntry, heEntry hentry);
 static void heBuiltinHelp(heEntry hentry, int /*br*/)
 {
   char* node = omStrDup(hentry != NULL && *(hentry->key) != '\0' ?
                        hentry->key : "Top");
-  singular_manual(node,(hentry != NULL) && *(hentry->url)!='\0');
+  singular_manual(node,(hentry != NULL) && *(hentry->url)!='\0',hentry);
   omFree(node);
 }
 
@@ -1095,7 +1119,7 @@ static int show(unsigned long offset, char *close)
 }
 
 /*************************************************/
-static int singular_manual(char *str, BOOLEAN isIndexEntry)
+static int singular_manual(char *str, BOOLEAN isIndexEntry,heEntry hentry)
 { FILE *index=NULL;
   unsigned long offset;
   char *p,close=' ';
@@ -1130,6 +1154,39 @@ static int singular_manual(char *str, BOOLEAN isIndexEntry)
   while(!feof(index)
         && (fgets(buffer, BUF_LEN, index) != (char *)0)
         && (buffer[0] != FIN_INDEX));
+   char temp[256];temp[0]='\0';
+   char *htmldir = feResource('u'); /* always defined */
+   if (hentry != NULL && *(hentry->url) != '\0')
+   #ifdef HAVE_VSNPRINTF
+   {
+     #ifdef SINGULAR_4_2
+     snprintf(temp,256,"%s/%d-%d/%s", htmldir,
+       SINGULAR_VERSION/1000,
+       (SINGULAR_VERSION % 1000)/100,
+     #else
+     snprintf(temp,256,"%s/%d-%d-%d/%s", htmldir,
+       SINGULAR_VERSION/1000,
+       (SINGULAR_VERSION % 1000)/100,
+       (SINGULAR_VERSION % 100)/10,
+     #endif
+       hentry->url);
+   }
+   #else
+   {
+     #ifdef SINGULAR_4_2
+     sprintf(temp,"%s/%d-%d/%s", htmldir,
+     SINGULAR_VERSION/1000,
+     (SINGULAR_VERSION % 1000)/100,
+     #else
+     sprintf(temp,"%s/%d-%d-%d/%s", htmldir,
+     SINGULAR_VERSION/1000,
+     (SINGULAR_VERSION % 1000)/100,
+     (SINGULAR_VERSION % 100)/10,
+     #endif
+     hentry->url);
+   }
+   #endif
+   Print("url: %s\n",temp);
 
   while(!feof(index))
   {
