@@ -30,6 +30,8 @@
 
 VAR char fe_promptstr[] ="  ";
 VAR FILE *File_Profiling=NULL;
+VAR FILE *File_Log=NULL;
+VAR BOOLEAN File_Log_written=FALSE;
 
 // line buffer for reading:
 // minimal value for MAX_FILE_BUFFER: 4*4096 - see Tst/Long/gcd0_l.tst
@@ -459,7 +461,7 @@ static int fePrintEcho(char *anf, char */*b*/)
       do
       {
         c=fgetc(stdin);
-	if (c=='n') traceit_stop=1;
+        if (c=='n') traceit_stop=1;
       }
       while(c!='\n');
     }
@@ -575,7 +577,8 @@ int feReadLine(char* b, int l)
       currentVoice->fptr=0;
       if (currentVoice->buffer==NULL)
       {
-        currentVoice->buffer=(char *)omAlloc(MAX_FILE_BUFFER-sizeof(ADDRESS));
+        currentVoice->buff_size=MAX_FILE_BUFFER-sizeof(ADDRESS);
+        currentVoice->buffer=(char *)omAlloc(currentVoice->buff_size);
         omMarkAsStaticAddr(currentVoice->buffer);
       }
     }
@@ -587,7 +590,7 @@ int feReadLine(char* b, int l)
       feShowPrompt();
       s=fe_fgets_stdin(fe_promptstr,
                        &(currentVoice->buffer[offset]),
-                       omSizeOfAddr(currentVoice->buffer)-1-offset);
+                       currentVoice->buff_size-1-offset);
       //int i=0;
       //if (s!=NULL)
       //  while((s[i]!='\0') /*&& (i<MAX_FILE_BUFFER)*/) {s[i] &= (char)127;i++;}
@@ -615,6 +618,11 @@ int feReadLine(char* b, int l)
     {
       fputs(s,feProtFile);
     }
+    if (File_Log!=NULL)
+    {
+      File_Log_written=TRUE;
+      fputs(s,File_Log);
+    }
     int rc=fePrintEcho(s,b)+1;
     //s[strlen(s)+1]='\0'; add an second \0 at the end of the string
     s[rc]='\0';
@@ -624,7 +632,7 @@ int feReadLine(char* b, int l)
     {
       s[rc]='\0';
       offset+=rc;
-      if (offset<(int)omSizeOfAddr(currentVoice->buffer)) goto NewRead;
+      if (offset<currentVoice->buff_size) goto NewRead;
     }
     goto NewBuff;
   }
