@@ -17,6 +17,10 @@
 #include "Singular/fevoices.h"
 #include "Singular/lists.h"
 
+#ifndef SINGULAR_PATH_LENGTH
+#define SINGULAR_PATH_LENGTH 512
+#endif
+
 #include <ctype.h>
 
 #if SIZEOF_LONG == 8
@@ -76,7 +80,7 @@ BOOLEAN iiGetLibStatus(const char *lib)
 
   char *plib = iiConvName(lib);
   hl = basePack->idroot->get(plib,0);
-  omFree(plib);
+  omFreeBinAddr(plib);
   if((hl==NULL) ||(IDTYP(hl)!=PACKAGE_CMD))
   {
     return FALSE;
@@ -192,7 +196,7 @@ char * iiProcArgs(char *e,BOOLEAN withParenth)
 */
 char* iiGetLibProcBuffer(procinfo *pi, int part )
 {
-  char buf[256], *s = NULL, *p;
+  char buf[SINGULAR_PATH_LENGTH], *s = NULL, *p;
   long procbuflen;
 
   FILE * fp = feFopen( pi->libname, "rb", NULL, TRUE );
@@ -658,7 +662,7 @@ ideal ii_CallProcId2Id(const char *lib,const char *proc, ideal arg, const ring R
 {
   char *plib = iiConvName(lib);
   idhdl h=ggetid(plib);
-  omFree(plib);
+  omFreeBinAddr(plib);
   if (h==NULL)
   {
     BOOLEAN bo=iiLibCmd(lib,TRUE,TRUE,FALSE);
@@ -677,7 +681,7 @@ int ii_CallProcId2Int(const char *lib,const char *proc, ideal arg, const ring R)
 {
   char *plib = iiConvName(lib);
   idhdl h=ggetid(plib);
-  omFree(plib);
+  omFreeBinAddr(plib);
   if (h==NULL)
   {
     BOOLEAN bo=iiLibCmd(lib,TRUE,TRUE,FALSE);
@@ -903,14 +907,14 @@ BOOLEAN iiLibCmd( const char *newlib, BOOLEAN autoexport, BOOLEAN tellerror, BOO
   {
     if(IDTYP(pl)!=PACKAGE_CMD)
     {
-      omFree(plib);
+      omFreeBinAddr(plib);
       WarnS("not of type package.");
       fclose(fp);
       return TRUE;
     }
     if (!force)
     {
-      omFree(plib);
+      omFreeBinAddr(plib);
       return FALSE;
     }
   }
@@ -1207,20 +1211,20 @@ BOOLEAN load_modules_aux(const char *newlib, char *fullname, BOOLEAN autoexport)
     if(IDPACKAGE(pl)->language==LANG_C)
     {
       if (BVERBOSE(V_LOAD_LIB)) Warn( "%s already loaded as package", newlib);
-      omFree(plib);
+      omFreeBinAddr(plib);
       return FALSE;
     }
     else if(IDPACKAGE(pl)->language==LANG_MIX)
     {
       if (BVERBOSE(V_LOAD_LIB)) Warn( "%s contain binary parts, cannot load", newlib);
-      omFree(plib);
+      omFreeBinAddr(plib);
       return FALSE;
     }
   }
   else
   {
     pl = enterid( plib,0, PACKAGE_CMD, &IDROOT, TRUE );
-    omFree(plib); /* enterid copied plib*/
+    omFreeBinAddr(plib); /* enterid copied plib*/
     IDPACKAGE(pl)->libname=omStrDup(newlib);
   }
   IDPACKAGE(pl)->language = LANG_C;
@@ -1309,7 +1313,7 @@ BOOLEAN load_builtin(const char *newlib, BOOLEAN autoexport, SModulFunc_t init)
     if(IDPACKAGE(pl)->language==LANG_C)
     {
       if (BVERBOSE(V_LOAD_LIB)) Warn( "(builtin) %s already loaded", newlib);
-      omFree(plib);
+      omFreeBinAddr(plib);
       return FALSE;
     }
   }
@@ -1318,7 +1322,7 @@ BOOLEAN load_builtin(const char *newlib, BOOLEAN autoexport, SModulFunc_t init)
     pl = enterid( plib,0, PACKAGE_CMD, &IDROOT, TRUE );
     IDPACKAGE(pl)->libname=omStrDup(newlib);
   }
-  omFree(plib);
+  omFreeBinAddr(plib);
   IDPACKAGE(pl)->language = LANG_C;
 
   IDPACKAGE(pl)->handle=(void *)NULL;
@@ -1365,10 +1369,10 @@ void module_help_proc(const char *newlib,const char *p, const char *help)
   {
     package s=currPack;
     currPack=IDPACKAGE(pl);
-    char buff[256];
-    buff[255]='\0';
-    strncpy(buff,p,255);
-    strncat(buff,"_help",255-strlen(p));
+    char buff[SINGULAR_PATH_LENGTH];
+    buff[SINGULAR_PATH_LENGTH-1]='\0';
+    strncpy(buff,p,SINGULAR_PATH_LENGTH-1);
+    strncat(buff,"_help",SINGULAR_PATH_LENGTH-1-strlen(p));
     idhdl h=enterid(buff,0,STRING_CMD,&IDROOT,FALSE);
     IDSTRING(h)=omStrDup(help);
     currPack=s;
@@ -1516,7 +1520,7 @@ void libstack::push(const char */*p*/, char *libn)
 libstackv libstack::pop(const char */*p*/)
 {
   libstackv ls = this;
-  //omFree((ADDRESS)ls->libname);
+  omFree((ADDRESS)ls->libname);
   library_stack = ls->next;
   omFreeBin((ADDRESS)ls,  libstack_bin);
   return(library_stack);
