@@ -985,7 +985,7 @@ ideal idLiftStd (ideal  h1, matrix* T, tHomog hi, ideal * S, GbVariant alg,
   if (S!=NULL) { lift3=TRUE; idDelete(S); }
   if (idIs0(h1))
   {
-    *T=mpNew(1,0);
+    *T=mpNew(1,IDELEMS(h1));
     if (lift3)
     {
       *S=idFreeModule(IDELEMS(h1));
@@ -1095,7 +1095,13 @@ static void idLift_setUnit(int e_mod, matrix *unit)
 *computes a representation of the generators of submod with respect to those
 * of mod
 */
-
+/// represents the generators of submod in terms of the generators of mod
+/// (Matrix(SM)*U-Matrix(rest)) = Matrix(M)*Matrix(result)
+/// goodShape: maximal non-zero index in generators of SM <= that of M
+/// isSB: generators of M form a Groebner basis
+/// divide: allow SM not to be a submodule of M
+/// U is an diagonal matrix of units (non-constant only in local rings)
+/// rest is: 0 if SM in M, SM if not divide, NF(SM,std(M)) if divide
 ideal idLift(ideal mod, ideal submod,ideal *rest, BOOLEAN goodShape,
              BOOLEAN isSB, BOOLEAN divide, matrix *unit, GbVariant alg)
 {
@@ -1624,11 +1630,7 @@ ideal idElimination (ideal h1,poly delVar,intvec *hilb, GbVariant alg)
 #endif
   hom=(tHomog)idHomModule(h1,NULL,&w); //sets w to weight vector or NULL
   h3=idInit(16,h1->rank);
-  for (k=0;; k++)
-  {
-    if (origR->order[k]!=0) ordersize++;
-    else break;
-  }
+  ordersize=rBlocks(origR)+1;
 #if 0
   if (rIsPluralRing(origR)) // we have too keep the odering: it may be needed
                             // for G-algebra
@@ -1727,7 +1729,17 @@ ideal idElimination (ideal h1,poly delVar,intvec *hilb, GbVariant alg)
       block0[k+1] = origR->block0[k];
       block1[k+1] = origR->block1[k];
       ord[k+1] = origR->order[k];
-      if (origR->wvhdl[k]!=NULL) wv[k+1] = (int*) omMemDup(origR->wvhdl[k]);
+      if (origR->wvhdl[k]!=NULL)
+      #ifdef HAVE_OMALLOC
+        wv[k+1] = (int*) omMemDup(origR->wvhdl[k]);
+      #else
+      {
+        int l=(origR->block1[k]-origR->block0[k]+1)*sizeof(int);
+        if (origR->order[k]==ringorder_a64) l*=2;
+        wv[k+1]=(int*)omalloc(l);
+        memcpy(wv[k+1],origR->wvhdl[k],l);
+      }
+      #endif
     }
     block0[0] = 1;
     block1[0] = rVar(origR);
