@@ -7,6 +7,7 @@
 
 #include "kernel/mod2.h"
 #include <errno.h>
+#include <stdio.h>
 
 // ----------------------------------------
 // system settings:
@@ -21,10 +22,7 @@
 #include "misc/options.h"
 
 #include "kernel/oswrapper/feread.h"
-
-#ifdef HAVE_STATIC
-#undef HAVE_DYN_RL
-#endif
+#include "reporter/reporter.h"
 
 #if defined(HAVE_DYN_RL)
 #include <unistd.h>
@@ -53,8 +51,8 @@ extern char *iiArithGetCmd(int);
 extern "C"
 char *command_generator (char *text, int state)
 {
-  static int list_index, len;
-  static idhdl h;
+  STATIC_VAR int list_index, len;
+  STATIC_VAR idhdl h;
   const char *name;
 
   /* If this is a new word to complete, initialize now.  This includes
@@ -105,6 +103,7 @@ char *command_generator (char *text, int state)
 // #undef READLINE_READLINE_H_OK
 
 extern "C" {
+  VAR BOOLEAN using_history_called=FALSE;
   typedef char * (*RL_PROC)(const char*,int);
   #ifdef READLINE_READLINE_H_OK
     #include <readline/readline.h>
@@ -125,14 +124,14 @@ extern "C" {
   #endif
   #ifndef READLINE_READLINE_H_OK
     /* declare everything we need explicitely and do not rely on includes */
-    extern char * rl_readline_name;
-    extern char *rl_line_buffer;
+    EXTERN_VAR char * rl_readline_name;
+    EXTERN_VAR char *rl_line_buffer;
     char *rl_filename_completion_function(const char*, int);
     typedef char **CPPFunction ();
 
     extern char ** rl_completion_matches (const char*, RL_PROC);
-    extern CPPFunction * rl_attempted_completion_function;
-    extern FILE * rl_outstream;
+    EXTERN_VAR CPPFunction * rl_attempted_completion_function;
+    EXTERN_VAR FILE * rl_outstream;
     extern char * readline (const char *);
     extern void add_history (char *);
     extern int write_history ();
@@ -163,16 +162,16 @@ extern "C"
   int fe_init_dyn_rl();
   char *(*fe_filename_completion_function)(); /* 3 */
   char *(* fe_readline) (char *);             /* 4 */
-  void (*fe_add_history) (char *);            /* 5 */
-  char ** fe_rl_readline_name;                /* 6 */
-  char **fe_rl_line_buffer;                   /* 7 */
+  VAR void (*fe_add_history) (char *);            /* 5 */
+  VAR char ** fe_rl_readline_name;                /* 6 */
+  VAR char **fe_rl_line_buffer;                   /* 7 */
   char **(*fe_completion_matches)(...);          /* 8 */
-  CPPFunction **fe_rl_attempted_completion_function; /* 9 */
-  FILE ** fe_rl_outstream;                    /* 10 */
-  int (*fe_write_history) ();                 /* 11 */
-  int (*fe_history_total_bytes) ();           /* 12 */
-  void (*fe_using_history) ();                /* 13 */
-  int (*fe_read_history) (char *);            /* 14 */
+  VAR CPPFunction **fe_rl_attempted_completion_function; /* 9 */
+  VAR FILE ** fe_rl_outstream;                    /* 10 */
+  VAR int (*fe_write_history) ();                 /* 11 */
+  VAR int (*fe_history_total_bytes) ();           /* 12 */
+  VAR void (*fe_using_history) ();                /* 13 */
+  VAR int (*fe_read_history) (char *);            /* 14 */
 
 }
 #endif
@@ -346,7 +345,7 @@ static char * fe_fgets_stdin_init(const char *pr,char *s, int size)
 {
 #if (defined(HAVE_READLINE) || defined(HAVE_LIBREADLINE)) && !defined(HAVE_DYN_RL) && !defined(HAVE_FEREAD)
   /* Allow conditional parsing of the ~/.inputrc file. */
-  rl_readline_name = "Singular";
+  rl_readline_name = (char*)"Singular";
   /* Tell the completer that we want a crack first. */
 #ifdef USE_READLINE4
   rl_attempted_completion_function = (rl_completion_func_t *)singular_completion;
@@ -365,12 +364,15 @@ static char * fe_fgets_stdin_init(const char *pr,char *s, int size)
     #endif
   }
 
+  using_history_called=FALSE;
   if(isatty(fileno(stdin)))
   {
     /* try to read a history */
+    using_history_called=TRUE;
     using_history();
     char *p = getenv("SINGULARHIST");
-    if (p != NULL)
+    if (p==NULL) p=SINGULARHIST_FILE;
+    if (strlen(p) != 0)
     {
       read_history (p);
     }
@@ -386,6 +388,7 @@ static char * fe_fgets_stdin_init(const char *pr,char *s, int size)
 #ifdef HAVE_DYN_RL
   /* do dynamic loading */
   int res=fe_init_dyn_rl();
+  using_history_called=FALSE;
   if (res!=0)
   {
     //if (res==1)
@@ -409,6 +412,7 @@ static char * fe_fgets_stdin_init(const char *pr,char *s, int size)
     *fe_rl_attempted_completion_function = (CPPFunction *)singular_completion;
     /* try to read a history */
     (*fe_using_history)();
+    using_history_called=TRUE;
     char *p = getenv("SINGULARHIST");
     if (p != NULL)
     {
