@@ -22,6 +22,11 @@ static number nrzMult (number a, number b, const coeffs)
   return (number) erg;
 }
 
+static void nrzInpMult (number &a, number b, const coeffs)
+{
+  mpz_mul((mpz_ptr) a, (mpz_ptr) a, (mpz_ptr) b);
+}
+
 /*
  * Give the smallest non unit k, such that a * x = k = b * y has a solution
  */
@@ -124,7 +129,6 @@ void nrzDelete(number *a, const coeffs)
 
 static number nrzCopy(number a, const coeffs)
 {
-  if (a==NULL) return NULL;
   mpz_ptr erg = (mpz_ptr) omAllocBin(gmp_nrz_bin);
   mpz_init_set(erg, (mpz_ptr) a);
   return (number) erg;
@@ -162,6 +166,11 @@ static number nrzAdd (number a, number b, const coeffs)
   return (number) erg;
 }
 
+static void nrzInpAdd (number &a, number b, const coeffs)
+{
+  mpz_add((mpz_ptr) a, (mpz_ptr) a, (mpz_ptr) b);
+}
+
 static number nrzSub (number a, number b, const coeffs)
 {
   mpz_ptr erg = (mpz_ptr) omAllocBin(gmp_nrz_bin);
@@ -303,6 +312,7 @@ static number nrzSmallestQuotRem (number a, number b, number * r, const coeffs )
   return (number) qq;
 }
 
+/*
 static number nrzQuotRem (number a, number b, number * r, const coeffs )
 {
   mpz_ptr qq = (mpz_ptr) omAllocBin(gmp_nrz_bin);
@@ -321,6 +331,7 @@ static number nrzQuotRem (number a, number b, number * r, const coeffs )
   }
   return (number) qq;
 }
+*/
 
 static number nrzIntMod (number a,number b, const coeffs)
 {
@@ -373,6 +384,32 @@ static number nrzMapQ(number from, const coeffs src, const coeffs /*dst*/)
   return (number) erg;
 }
 
+static number nrzMaplongR(number from, const coeffs src, const coeffs dst)
+{
+  gmp_float *ff=(gmp_float*)from;
+  if (mpf_fits_slong_p(ff->t))
+  {
+    long l=mpf_get_si(ff->t);
+    return nrzInit(l,dst);
+  }
+  char *out=floatToStr(*(gmp_float*)from, src->float_len);
+  char *p=strchr(out,'.');
+  *p='\0';
+  mpz_ptr erg = (mpz_ptr) omAllocBin(gmp_nrz_bin);
+  mpz_init(erg);
+  if (out[0]=='-')
+  {
+    mpz_set_str(erg,out+1,10);
+    mpz_mul_si(erg, erg, -1);
+  }
+  else
+  {
+    mpz_set_str(erg,out,10);
+  }
+  omFree( (void *)out );
+  return (number) erg;
+}
+
 static nMapFunc nrzSetMap(const coeffs src, const coeffs /*dst*/)
 {
   /* dst = currRing */
@@ -398,6 +435,10 @@ static nMapFunc nrzSetMap(const coeffs src, const coeffs /*dst*/)
   {
     return nrzMapQ;
   }
+  if (nCoeff_is_long_R(src))
+  {
+    return nrzMaplongR;
+  }
   return NULL;      // default
 }
 
@@ -625,8 +666,10 @@ BOOLEAN nrzInitChar(coeffs r,  void *)
   r->cfCoeffName = nrzCoeffName;
   //r->cfKillChar = ndKillChar;
   r->cfMult  = nrzMult;
+  r->cfInpMult  = nrzInpMult;
   r->cfSub   = nrzSub;
   r->cfAdd   = nrzAdd;
+  r->cfInpAdd   = nrzInpAdd;
   r->cfDiv   = nrzDiv;
   r->cfIntMod= nrzIntMod;
   r->cfExactDiv= nrzExactDiv;
