@@ -206,6 +206,12 @@ static number nrnMult(number a, number b, const coeffs r)
   return (number) erg;
 }
 
+static void nrnInpMult(number &a, number b, const coeffs r)
+{
+  mpz_mul((mpz_ptr)a, (mpz_ptr)a, (mpz_ptr) b);
+  mpz_mod((mpz_ptr)a, (mpz_ptr)a, r->modNumber);
+}
+
 static void nrnPower(number a, int i, number * result, const coeffs r)
 {
   mpz_ptr erg = (mpz_ptr)omAllocBin(gmp_nrz_bin);
@@ -223,6 +229,12 @@ static number nrnAdd(number a, number b, const coeffs r)
   return (number) erg;
 }
 
+static void nrnInpAdd(number &a, number b, const coeffs r)
+{
+  mpz_add((mpz_ptr)a, (mpz_ptr)a, (mpz_ptr) b);
+  mpz_mod((mpz_ptr)a, (mpz_ptr)a, r->modNumber);
+}
+
 static number nrnSub(number a, number b, const coeffs r)
 {
   mpz_ptr erg = (mpz_ptr)omAllocBin(gmp_nrz_bin);
@@ -503,13 +515,31 @@ static BOOLEAN nrnGreaterZero(number k, const coeffs cf)
     {
       return TRUE;
     }
+    #if 0
     mpz_t ch2; mpz_init_set(ch2, cf->modBase);
-    mpz_sub_ui(ch2,ch2,1);
+    mpz_sub_ui(ch2,ch2,1); //cf->modBase is odd
     mpz_divexact_ui(ch2,ch2,2);
     if (mpz_cmp(ch2,(mpz_ptr)k)<0)
+    {
+      mpz_clear(ch2);
       return FALSE;
+    }
+    mpz_clear(ch2);
+    #endif
+  }
+  #if 0
+  else
+  {
+    mpz_t ch2; mpz_init_set(ch2, cf->modBase);
+    mpz_tdiv_q_ui(ch2,ch2,2);
+    if (mpz_cmp(ch2,(mpz_ptr)k)<0)
+    {
+      mpz_clear(ch2);
+      return FALSE;
+    }
     mpz_clear(ch2);
   }
+  #endif
   return 0 < mpz_sgn1((mpz_ptr)k);
 }
 
@@ -749,7 +779,7 @@ static number nrnMapZ(number from, const coeffs src, const coeffs dst)
   return nrnMapQ(from,src,dst);
 }
 #endif
-void nrnWrite (number a, const coeffs cf)
+void nrnWrite (number a, const coeffs /*cf*/)
 {
   char *s,*z;
   if (a==NULL)
@@ -760,29 +790,8 @@ void nrnWrite (number a, const coeffs cf)
   {
     int l=mpz_sizeinbase((mpz_ptr) a, 10) + 2;
     s=(char*)omAlloc(l);
-    if (cf->is_field)
-    {
-      mpz_t ch2; mpz_init_set(ch2, cf->modBase);
-      mpz_sub_ui(ch2,ch2,1);
-      mpz_divexact_ui(ch2,ch2,2);
-      if ((mpz_cmp_ui(cf->modBase,2)!=0) && (mpz_cmp(ch2,(mpz_ptr)a)<0))
-      {
-        mpz_sub(ch2,(mpz_ptr)a,cf->modBase);
-        z=mpz_get_str(s,10,ch2);
-        StringAppendS(z);
-      }
-      else
-      {
-        z=mpz_get_str(s,10,(mpz_ptr) a);
-        StringAppendS(z);
-      }
-      mpz_clear(ch2);
-    }
-    else
-    {
-      z=mpz_get_str(s,10,(mpz_ptr) a);
-      StringAppendS(z);
-    }
+    z=mpz_get_str(s,10,(mpz_ptr) a);
+    StringAppendS(z);
     omFreeSize((ADDRESS)s,l);
   }
 }
@@ -1008,8 +1017,10 @@ BOOLEAN nrnInitChar (coeffs r, void* p)
   r->cfSize        = nrnSize;
   r->cfInt         = nrnInt;
   r->cfAdd         = nrnAdd;
+  r->cfInpAdd      = nrnInpAdd;
   r->cfSub         = nrnSub;
   r->cfMult        = nrnMult;
+  r->cfInpMult     = nrnInpMult;
   r->cfDiv         = nrnDiv;
   r->cfAnn         = nrnAnn;
   r->cfIntMod      = nrnMod;
